
# q-expansion of newform 510.2.a.g, downloaded from the LMFDB on 12 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 510
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [1], [1], [2], [0], [-4], [-1], [-4], [0], [6], [-4], [2], [0], [2], [0], [-6], [6], [-10], [2], [-6], [-16], [8], [0], [6], [-4], [12], [-16], [12], [2], [-6], [8], [12], [-18], [-4], [12], [8], [20], [-16], [12], [-6], [18], [-22], [12], [-4], [-6], [20], [-4], [8], [-12], [14], [6], [24], [26], [30], [-30], [0], [6], [8], [-10], [6], [-4], [6], [2], [6], [20], [-30], [32], [32], [12], [26], [-30], [-12], [14], [32], [-4], [24], [-12], [2], [-12], [-10], [-36], [-10], [-18], [-10], [-16], [24], [-12], [-10], [0], [8], [-36], [-30], [2], [18], [-28], [-24], [12], [-36], [-34], [38], [20], [-6], [24], [-30], [20], [-34], [24], [18], [12], [-34], [-22], [-40], [30], [-28], [8], [-12], [-40], [48], [30], [-30], [26], [-4], [-6], [-12], [-4], [24], [38], [-6], [-16], [8], [-40], [36], [32], [20], [18], [14], [-6], [32], [30], [24], [20], [18], [14], [-12], [50], [54], [38], [-18], [-40], [-48], [14], [-12], [-34], [-24], [-16], [18], [-16], [-48], [-10], [42], [36], [30], [-4], [-30], [-30], [-60], [-52], [-46]]
